<?php

declare(strict_types=1);

namespace Dedi\SyliusImportExportPlugin\ShippingMethod\Exporter;

use Dedi\SyliusImportExportPlugin\ShippingMethod\ShippingMethodRuleTrait;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Sylius\Component\Core\Model\ShippingMethodInterface;
use Sylius\Component\Core\Shipping\Checker\Rule\OrderTotalLessThanOrEqualRuleChecker;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Sylius\Component\Shipping\Checker\Rule\TotalWeightGreaterThanOrEqualRuleChecker;
use Sylius\Component\Shipping\Checker\Rule\TotalWeightLessThanOrEqualRuleChecker;
use Sylius\Component\Taxation\Calculator\CalculatorInterface;
use Sylius\Component\Taxation\Model\TaxRateInterface;
use Sylius\Component\Taxation\Resolver\TaxRateResolverInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

class ShippingMethodDataProvider implements ShippingMethodDataProviderInterface
{
    use ShippingMethodRuleTrait;

    private readonly RepositoryInterface $shippingMethodRepository;

    private readonly TranslatorInterface $translator;

    private readonly TaxRateResolverInterface $taxRateResolver;

    private readonly CalculatorInterface $calculator;

    public function __construct(
        RepositoryInterface $shippingMethodRepository,
        TranslatorInterface $translator,
        TaxRateResolverInterface $taxRateResolver,
        CalculatorInterface $calculator
    ) {
        $this->shippingMethodRepository = $shippingMethodRepository;
        $this->translator = $translator;
        $this->taxRateResolver = $taxRateResolver;
        $this->calculator = $calculator;
    }

    public function getData(array $criteria = []): Collection
    {
        $data = new ArrayCollection();

        $headers = $this->getHeaders();

        /** @var ShippingMethodInterface[] $shippingMethods */
        $shippingMethods = $this->shippingMethodRepository->findAll();
        foreach ($shippingMethods as $shippingMethod) {
            $rowData = $this->getRowData($shippingMethod);
            $data->add(array_combine($headers, $rowData));
        }

        return $data;
    }

    public function getRowData(ShippingMethodInterface $shippingMethod): array
    {
        $taxRate = $this->taxRateResolver->resolve($shippingMethod);

        $minWeightRule = $this->getRule($shippingMethod, TotalWeightGreaterThanOrEqualRuleChecker::TYPE);
        $maxWeightRule = $this->getRule($shippingMethod, TotalWeightLessThanOrEqualRuleChecker::TYPE);
        $maxOrderTotal = $this->getRule($shippingMethod, OrderTotalLessThanOrEqualRuleChecker::TYPE);

        $pricesWithTax = [];
        $maxOrderAmountsWithoutTax = [];
        $channelCodes = [];

        foreach ($shippingMethod->getChannels() as $channel) {
            $channelCode = $channel->getCode();
            $channelCodes[] = $channelCode;

            $priceWithTax = $shippingMethod->getConfiguration()[$channelCode]['amount'];
            $priceWithTax /= 100;

            $pricesWithTax[] = $priceWithTax;

            $maxOrderAmountWithoutTax = '';
            if (null !== $maxOrderTotal) {
                $maxOrderAmountWithTax = $maxOrderTotal->getConfiguration()[$channelCode]['amount'];
                $maxOrderAmountWithoutTax = $this->getAmountWithoutTax($maxOrderAmountWithTax, $taxRate);
                $maxOrderAmountWithoutTax /= 100;
            }
            $maxOrderAmountsWithoutTax[] = $maxOrderAmountWithoutTax;
        }

        if (\count(array_unique($pricesWithTax)) === 1) {
            $pricesWithTax = [$pricesWithTax[0]];
        }
        if (\count(array_unique($maxOrderAmountsWithoutTax)) === 1) {
            $maxOrderAmountsWithoutTax = [$maxOrderAmountsWithoutTax[0]];
        }

        return [
            $shippingMethod->getPosition(),
            $shippingMethod->getCode(),
            $shippingMethod->getName(),
            null !== $shippingMethod->getZone() ? $shippingMethod->getZone()->getCode() : '',
            implode(' | ', $pricesWithTax),
            null !== $minWeightRule ? $minWeightRule->getConfiguration()['weight'] : '',
            null !== $maxWeightRule ? $maxWeightRule->getConfiguration()['weight'] : '',
            implode(' | ', $maxOrderAmountsWithoutTax),
            $shippingMethod->getDescription(),
            implode(' | ', $channelCodes),
        ];
    }

    public function getHeaders(): array
    {
        return [
            'position' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.position'),
            'code' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.code'),
            'name' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.name'),
            'zone' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.zone'),
            'price_including_tax' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.price_including_tax'),
            'min_weight' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.min_weight'),
            'max_weight' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.max_weight'),
            'max_cart_amount_excluding_tax' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.max_cart_amount_excluding_tax'),
            'description' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.description'),
            'channel' => $this->translator->trans('dedi_sylius_import_export.export.shipping_method.channel'),
        ];
    }

    private function getAmountWithoutTax(float $amountWithTax, ?TaxRateInterface $taxRate): float
    {
        if (null === $taxRate || false === $taxRate->isIncludedInPrice()) {
            return $amountWithTax;
        }

        $priceTaxAmount = $this->calculator->calculate($amountWithTax, $taxRate);

        return $amountWithTax - $priceTaxAmount;
    }
}
